"""
Additional Netlab datatypes.
"""

from .common import HDREvent, HDRResult
from .pod import PCCloneSpec
from .vm import VMAlloc, VMAllocHostInfo, VMAllocTotal

__all__ = (
    'VMAlloc', 'VMAllocTotal', 'VMAllocHostInfo',

    'PCCloneSpec',

    'HDREvent', 'HDRResult',
)
