from enum import Enum


class FreeAlarm(Enum):
    "These values are triggered when a disk's free space has dropped below a determined threshold."
    CLEAR = 'CLEAR'
    "The disk has adequate free space."
    WARN = 'WARN'
    "The disk has reached free space warning"
    MAJOR = 'MAJOR'
    "The disk is nearly out of free space."
    CRITICAL = 'CRITICAL'
    "The disk is out of free space."
